package com.tuoheng.miniprogram.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tuoheng.common.core.common.BaseServiceImpl;
import com.tuoheng.common.core.utils.DateUtils;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.SecurityUserUtils;
import com.tuoheng.common.core.utils.StringUtils;
import com.tuoheng.miniprogram.dao.*;
import com.tuoheng.miniprogram.entity.InspectionFile;
import com.tuoheng.miniprogram.entity.User;
import com.tuoheng.miniprogram.entity.WorkOrder;
import com.tuoheng.miniprogram.entity.WorkOrderFile;
import com.tuoheng.miniprogram.entity.dto.WorkOrderDto;
import com.tuoheng.miniprogram.enums.InspectionFileStatusEnum;
import com.tuoheng.miniprogram.enums.WorkOrderEnum;
import com.tuoheng.miniprogram.enums.WorkOrderStatusEnum;
import com.tuoheng.miniprogram.service.IWorkOrderService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @Author ChengWang
 * @Date 2022/12/6
 */
@Service
@Slf4j
public class WorkOrderServiceImpl extends BaseServiceImpl<WorkOrderMapper, WorkOrder> implements IWorkOrderService {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private WorkOrderMapper workOrderMapper;

    @Autowired
    private WorkOrderFileMapper workOrderFileMapper;

    @Autowired
    private InspectionFileMapper inspectionFileMapper;

    @Autowired
    private DeptMapper deptMapper;

    @Override
    public JsonResult addInfo(WorkOrderDto dto) {
        if(StringUtils.isEmpty(dto.getTenantId())){
            return JsonResult.error(WorkOrderEnum.TENANT_ID_IS_NULL.getCode(),WorkOrderEnum.TENANT_ID_IS_NULL.getMsg());
        }
        if(StringUtils.isEmpty(dto.getInspectionFileId())) {
            return JsonResult.error(WorkOrderEnum.INSPECTION_FILE_ID_IS_NULL.getCode(), WorkOrderEnum.INSPECTION_FILE_ID_IS_NULL.getMsg());
        }
//        if(StringUtils.isEmpty(dto.getUserId())){
//            return JsonResult.error(WorkOrderEnum.USER_ID_IS_NULL.getCode(),WorkOrderEnum.USER_ID_IS_NULL.getMsg());
//        }
        //String username = SecurityUserUtils.username();
        String username ="admin";
        if(StringUtils.isEmpty(username)){
            return JsonResult.error(WorkOrderEnum.USER_NAME_IS_NULL.getCode(),WorkOrderEnum.USER_NAME_IS_NULL.getMsg());
        }
        //根据用户id查询部门id
        User user = userMapper.selectOne(Wrappers.<User>lambdaQuery()
                .eq(User::getMark, 1).eq(User::getUsername,username));
        if(ObjectUtil.isNull(user)){
            return JsonResult.error(WorkOrderEnum.USER__IS_NOT_EXIST.getCode(),WorkOrderEnum.USER__IS_NOT_EXIST.getMsg());
        }
        String deptId = user.getDeptId();
        if(StringUtils.isEmpty(deptId)){
            return JsonResult.error(WorkOrderEnum.DEPT__ID_IS_NULL.getCode(),WorkOrderEnum.DEPT__ID_IS_NULL.getMsg());
        }
        WorkOrder workOrder = new WorkOrder();
        workOrder.setTenantId(dto.getTenantId());
        workOrder.setDeptId(deptId);
        //分配人员是当前登录用户
        workOrder.setDistributionId(user.getId());
        //工单编号 XJGD+日期+随机
        Date now = DateUtils.now();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        String str = sdf.format(now);
        String code = "XJGD"+str+(int)(Math.random()*10000);
        workOrder.setCode(code);
        workOrder.setStatus(WorkOrderStatusEnum.STATUS_ASSIGNED.getCode());
        workOrder.setAssignDeptId(deptId);
        workOrder.setAssignUser(username);
        workOrder.setAssignContact(user.getMobile());
        workOrder.setAssignTime(DateUtils.now());
        workOrder.setCreateUser(user.getId());
        workOrder.setCreateTime(DateUtils.now());
        workOrder.setMark(1);
        super.add(workOrder);
        //工单子表数据入库
        WorkOrderFile workOrderFile = new WorkOrderFile();
        workOrderFile.setTenantId(dto.getTenantId());
        workOrderFile.setInspectionFileId(dto.getInspectionFileId());
        workOrderFile.setWorkOrderId(workOrder.getId());
        int num = workOrderFileMapper.insert(workOrderFile);
        if(num<=0){
            return JsonResult.error();
        }
        //更新问题状态为工单已生成
        InspectionFile inspectionFile = inspectionFileMapper.selectById(dto.getInspectionFileId());
        inspectionFile.setStatus(InspectionFileStatusEnum.GENERATE_ORDER.getCode());
        int count = inspectionFileMapper.updateById(inspectionFile);
        if(count<=0){
            return JsonResult.error();
        }

        return JsonResult.success();
    }
}
